Engine.DebugLua(true)

local Objects    = {}
local Velocities = {}

-- new border extents
local MinX, MinY = 0, 0
local MaxX, MaxY = 5096, 5096
local Speed      = 1000  -- units per second

-- simple clamp
local function clamp(v, lo, hi)
    if v < lo then return lo end
    if v > hi then return hi end
    return v
end

function OnInit()
    math.randomseed(os.time())
    for i = 0, 99 do
        local tag = "Light_" .. i
        local obj = Engine.GetObjectByTag(tag)
        if obj then
            table.insert(Objects, obj)
            -- random direction
            local ang = math.random() * 2 * math.pi
            local vx = math.cos(ang) * Speed
            local vy = math.sin(ang) * Speed
            -- store velocity in plain table
            Velocities[obj] = { x = vx, y = vy }
        else
            Engine.LogError("Object '" .. tag .. "' not found!")
        end
    end
end

function OnUpdate(dt)
    for _, obj in ipairs(Objects) do
        local vel = Velocities[obj]
        local pos = obj:GetPosition()

        -- integrate
        local nx = pos.x + vel.x * dt
        local ny = pos.y + vel.y * dt

        -- bounce X against [MinX,MaxX]
        if nx < MinX or nx > MaxX then
            vel.x = -vel.x
            nx    = clamp(nx, MinX, MaxX)
        end

        -- bounce Y against [MinY,MaxY]
        if ny < MinY or ny > MaxY then
            vel.y = -vel.y
            ny    = clamp(ny, MinY, MaxY)
        end

        -- store back and apply
        Velocities[obj] = vel
        obj:SetPosition(Vector2(nx, ny))
    end
end
